@@ -51,36 +51,12 @@ database_opts = [
     cfg.IntOpt(
         'connection_recycle_time',
         default=3600,
-        deprecated_opts=[
-            cfg.DeprecatedOpt('idle_timeout',
-                              group="DATABASE"),
-            cfg.DeprecatedOpt('idle_timeout',
-                              group="database"),
-            cfg.DeprecatedOpt('sql_idle_timeout',
-                              group='DEFAULT'),
-            cfg.DeprecatedOpt('sql_idle_timeout',
-                              group='DATABASE'),
-            cfg.DeprecatedOpt('idle_timeout',
-                              group='sql')
-        ],
         help='Connections which have been present in the connection '
              'pool longer than this number of seconds will be replaced '
              'with a new one the next time they are checked out from '
              'the pool.'),
-    cfg.IntOpt('min_pool_size',
-               default=1,
-               deprecated_opts=[cfg.DeprecatedOpt('sql_min_pool_size',
-                                                  group='DEFAULT'),
-                                cfg.DeprecatedOpt('sql_min_pool_size',
-                                                  group='DATABASE')],
-               help='Minimum number of SQL connections to keep open in a '
-                    'pool.'),
     cfg.IntOpt('max_pool_size',
                default=5,
-               deprecated_opts=[cfg.DeprecatedOpt('sql_max_pool_size',
-                                                  group='DEFAULT'),
-                                cfg.DeprecatedOpt('sql_max_pool_size',
-                                                  group='DATABASE')],
                help='Maximum number of SQL connections to keep open in a '
                     'pool. Setting a value of 0 indicates no limit.'),
     cfg.IntOpt('max_retries',
@@ -146,6 +122,11 @@ database_opts = [
                     'error before error is '
                     'raised. Set to -1 to specify an infinite retry '
                     'count.'),
+    cfg.StrOpt('connection_parameters',
+               default='',
+               help='Optional URL parameters to append onto the connection '
+               'URL at connect time; specify as '
+               'param1=value1&param2=value2&...'),
 ]
 
 
