@@ -10,13 +10,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
 import contextlib
 import functools
+import inspect
 import operator
 import threading
 import warnings
 
+import debtcollector.moves
 import debtcollector.removals
 import debtcollector.renames
 from oslo_config import cfg
@@ -26,7 +27,7 @@ from oslo_db import exception
 from oslo_db import options
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import orm
-from oslo_db.sqlalchemy import utils
+from oslo_db import warning
 
 
 class _symbol(object):
@@ -119,6 +120,14 @@ class _Default(object):
             hasattr(conf_namespace, key)
 
 
+class AlreadyStartedError(TypeError):
+    """Raises when a factory is being asked to initialize a second time.
+
+    Subclasses :class:`.TypeError` for legacy support.
+
+    """
+
+
 class _TransactionFactory(object):
     """A factory for :class:`._TransactionContext` objects.
 
@@ -149,7 +158,8 @@ class _TransactionFactory(object):
             'thread_checkin': _Default(True),
             'json_serializer': _Default(None),
             'json_deserializer': _Default(None),
-            'logging_name': _Default(None)
+            'logging_name': _Default(None),
+            'connection_parameters': _Default(None)
         }
         self._maker_cfg = {
             'expire_on_commit': _Default(False),
@@ -164,22 +174,20 @@ class _TransactionFactory(object):
             'on_engine_create': [],
         }
 
-        # other options that are defined in oslo.db.options.database_opts
-        # but do not apply to the standard enginefacade arguments
-        # (most seem to apply to api.DBAPI).
+        # other options that are defined in oslo_db.options.database_opts
+        # or oslo_db.concurrency.tpool_opts but do not apply to the standard
+        # enginefacade arguments (most seem to apply to api.DBAPI).
         self._ignored_cfg = dict(
             (k, _Default(None)) for k in [
                 'db_max_retries', 'db_inc_retry_interval',
                 'use_db_reconnect',
-                'db_retry_interval', 'min_pool_size',
-                'db_max_retry_interval', 'backend'])
+                'db_retry_interval',
+                'db_max_retry_interval', 'backend', 'use_tpool'])
 
         self._started = False
         self._legacy_facade = None
         self._start_lock = threading.Lock()
 
-    @debtcollector.renames.renamed_kwarg(
-        "idle_timeout", "connection_recycle_time", replace=True)
     def configure_defaults(self, **kw):
         """Apply default configurational options.
 
@@ -219,6 +227,9 @@ class _TransactionFactory(object):
         :param connection_debug: engine logging level, defaults to 0. set to
          50 for INFO, 100 for DEBUG.
 
+        :param connection_parameters: additional parameters to append onto the
+         database URL query string, pass as "param1=value1&param2=value2&..."
+
         :param max_pool_size: max size of connection pool, uses CONF for
          default
 
@@ -285,8 +296,6 @@ class _TransactionFactory(object):
         """
         self._configure(True, kw)
 
-    @debtcollector.renames.renamed_kwarg(
-        "idle_timeout", "connection_recycle_time", replace=True)
     def configure(self, **kw):
         """Apply configurational options.
 
@@ -309,7 +318,8 @@ class _TransactionFactory(object):
     def _configure(self, as_defaults, kw):
 
         if self._started:
-            raise TypeError("this TransactionFactory is already started")
+            raise AlreadyStartedError(
+                "this TransactionFactory is already started")
         not_supported = []
         for k, v in kw.items():
             for dict_ in (
@@ -329,7 +339,7 @@ class _TransactionFactory(object):
             warnings.warn(
                 "Configuration option(s) %r not supported" %
                 sorted(not_supported),
-                exception.NotSupportedWarning
+                warning.NotSupportedWarning
             )
 
     def get_legacy_facade(self):
@@ -393,7 +403,8 @@ class _TransactionFactory(object):
             self._start()
         if mode is _WRITER:
             return self._writer_engine.connect()
-        elif self.synchronous_reader or mode is _ASYNC_READER:
+        elif mode is _ASYNC_READER or \
+                (mode is _READER and not self.synchronous_reader):
             return self._reader_engine.connect()
         else:
             return self._writer_engine.connect()
@@ -408,7 +419,8 @@ class _TransactionFactory(object):
             kw['bind'] = bind
         if mode is _WRITER:
             return self._writer_maker(**kw)
-        elif self.synchronous_reader or mode is _ASYNC_READER:
+        elif mode is _ASYNC_READER or \
+                (mode is _READER and not self.synchronous_reader):
             return self._reader_maker(**kw)
         else:
             return self._writer_maker(**kw)
@@ -457,6 +469,11 @@ class _TransactionFactory(object):
             if self._reader_engine is not self._writer_engine:
                 self._reader_engine.pool.dispose()
 
+    @property
+    def is_started(self):
+        """True if this :class:`._TransactionFactory` is already started."""
+        return self._started
+
     def _start(self, conf=False, connection=None, slave_connection=None):
         with self._start_lock:
             # self._started has been checked on the outside
@@ -613,7 +630,6 @@ class _TransactionContext(object):
                         # TODO(zzzeek) do we need save_and_reraise() here,
                         # or do newer eventlets not have issues?  we are using
                         # raw "raise" in many other places in oslo.db already
-                        # (and one six.reraise()).
                         raise
                     finally:
                         self.transaction = None
@@ -770,6 +786,11 @@ class _TransactionContextManager(object):
         """The :class:`._TransactionFactory` associated with this context."""
         return self._root._root_factory
 
+    @property
+    def is_started(self):
+        """True if this manager is already started."""
+        return self._factory.is_started
+
     def configure(self, **kw):
         """Apply configurational options to the factory.
 
@@ -959,7 +980,7 @@ class _TransactionContextManager(object):
         return self._clone(connection=True)
 
     @property
-    def async(self):
+    def async_(self):
         """Modifier to set a READER operation to ASYNC_READER."""
 
         if self._mode is _WRITER:
@@ -972,15 +993,18 @@ class _TransactionContextManager(object):
 
     def __call__(self, fn):
         """Decorate a function."""
-        argspec = utils.getargspec(fn)
+        argspec = inspect.getfullargspec(fn)
         if argspec.args[0] == 'self' or argspec.args[0] == 'cls':
             context_index = 1
         else:
             context_index = 0
+        context_kw = argspec.args[context_index]
 
         @functools.wraps(fn)
         def wrapper(*args, **kwargs):
-            context = args[context_index]
+            context = kwargs.get(context_kw, None)
+            if not context:
+                context = args[context_index]
 
             with self._transaction_scope(context):
                 return fn(*args, **kwargs)
@@ -1043,6 +1067,18 @@ class _TransactionContextManager(object):
                 transaction_contexts_by_thread.current = restore
 
 
+@property
+@debtcollector.moves.moved_property("async_")
+def async_compat(self):
+    return self.async_
+
+
+setattr(
+    _TransactionContextManager,
+    "async", async_compat
+)
+
+
 def _context_descriptor(attr=None):
     getter = operator.attrgetter(attr)
 
@@ -1235,7 +1271,7 @@ class LegacyEngineFacade(object):
         warnings.warn(
             "EngineFacade is deprecated; please use "
             "oslo_db.sqlalchemy.enginefacade",
-            exception.OsloDBDeprecationWarning,
+            warning.OsloDBDeprecationWarning,
             stacklevel=2)
         if _factory:
             self._factory = _factory
