@@ -19,20 +19,19 @@
 import collections
 from collections import abc
 import contextlib
-import inspect as pyinspect
 import itertools
 import logging
 import re
 
 from alembic.migration import MigrationContext
 from alembic.operations import Operations
+import debtcollector.removals
 from oslo_utils import timeutils
 import sqlalchemy
 from sqlalchemy import Boolean
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
 from sqlalchemy.engine import Connectable
-from sqlalchemy.engine import reflection
 from sqlalchemy.engine import url as sa_url
 from sqlalchemy import exc
 from sqlalchemy import func
@@ -442,8 +441,7 @@ def get_table(engine, name):
        with ForeignKey creation.
     """
     metadata = MetaData()
-    metadata.bind = engine
-    return Table(name, metadata, autoload=True)
+    return Table(name, metadata, autoload_with=engine)
 
 
 def _get_not_supported_column(col_name_col_instance, column_name):
@@ -477,43 +475,49 @@ def drop_old_duplicate_entries_from_table(engine, table_name,
     :param uc_column_names: Unique constraint columns
     """
     meta = MetaData()
-    meta.bind = engine
 
-    table = Table(table_name, meta, autoload=True)
+    table = Table(table_name, meta, autoload_with=engine)
     columns_for_group_by = [table.c[name] for name in uc_column_names]
 
     columns_for_select = [func.max(table.c.id)]
     columns_for_select.extend(columns_for_group_by)
 
     duplicated_rows_select = sqlalchemy.sql.select(
-        columns_for_select, group_by=columns_for_group_by,
-        having=func.count(table.c.id) > 1)
+        *columns_for_select,
+    ).group_by(
+        *columns_for_group_by
+    ).having(
+        func.count(table.c.id) > 1
+    )
 
-    for row in engine.execute(duplicated_rows_select).fetchall():
-        # NOTE(boris-42): Do not remove row that has the biggest ID.
-        delete_condition = table.c.id != row[0]
-        is_none = None  # workaround for pyflakes
-        delete_condition &= table.c.deleted_at == is_none
-        for name in uc_column_names:
-            delete_condition &= table.c[name] == row[name]
+    with engine.connect() as conn, conn.begin():
+        for row in conn.execute(duplicated_rows_select).fetchall():
+            # NOTE(boris-42): Do not remove row that has the biggest ID.
+            delete_condition = table.c.id != row[0]
+            is_none = None  # workaround for pyflakes
+            delete_condition &= table.c.deleted_at == is_none
+            for name in uc_column_names:
+                delete_condition &= table.c[name] == row._mapping[name]
 
-        rows_to_delete_select = sqlalchemy.sql.select(
-            [table.c.id]).where(delete_condition)
-        for row in engine.execute(rows_to_delete_select).fetchall():
-            LOG.info("Deleting duplicated row with id: %(id)s from table: "
-                     "%(table)s", dict(id=row[0], table=table_name))
+            rows_to_delete_select = sqlalchemy.sql.select(
+                table.c.id,
+            ).where(delete_condition)
+            for row in conn.execute(rows_to_delete_select).fetchall():
+                LOG.info(
+                    "Deleting duplicated row with id: %(id)s from table: "
+                    "%(table)s", dict(id=row[0], table=table_name))
 
-        if use_soft_delete:
-            delete_statement = table.update().\
-                where(delete_condition).\
-                values({
-                    'deleted': literal_column('id'),
-                    'updated_at': literal_column('updated_at'),
-                    'deleted_at': timeutils.utcnow()
-                })
-        else:
-            delete_statement = table.delete().where(delete_condition)
-        engine.execute(delete_statement)
+            if use_soft_delete:
+                delete_statement = table.update().\
+                    where(delete_condition).\
+                    values({
+                        'deleted': literal_column('id'),
+                        'updated_at': literal_column('updated_at'),
+                        'deleted_at': timeutils.utcnow()
+                    })
+            else:
+                delete_statement = table.delete().where(delete_condition)
+            conn.execute(delete_statement)
 
 
 def _get_default_deleted_value(table):
@@ -546,6 +550,12 @@ def _restore_indexes_on_deleted_columns(engine, table_name, indexes):
         new_index.create(engine)
 
 
+@debtcollector.removals.remove(
+    message='This API is intended for use with sqlalchemy-migrate, support '
+    'for which is deprecated for removal; it will be removed in a future '
+    'release',
+    version='10.1.0',
+)
 def change_deleted_column_type_to_boolean(engine, table_name,
                                           **col_name_col_instance):
     if engine.name == "sqlite":
@@ -555,15 +565,25 @@ def change_deleted_column_type_to_boolean(engine, table_name,
     table = get_table(engine, table_name)
 
     old_deleted = Column('old_deleted', Boolean, default=False)
-    old_deleted.create(table, populate_default=False)
+    table.metadata.bind = engine
+    try:
+        old_deleted.create(table, populate_default=False)
+    finally:
+        table.metadata.bind = None
 
-    table.update().\
-        where(table.c.deleted == table.c.id).\
-        values(old_deleted=True).\
-        execute()
+    with engine.connect() as conn, conn.begin():
+        conn.execute(
+            table.update().where(
+                table.c.deleted == table.c.id
+            ).values(old_deleted=True)
+        )
 
-    table.c.deleted.drop()
-    table.c.old_deleted.alter(name="deleted")
+    table.metadata.bind = engine
+    try:
+        table.c.deleted.drop()
+        table.c.old_deleted.alter(name="deleted")
+    finally:
+        table.metadata.bind = None
 
     _restore_indexes_on_deleted_columns(engine, table_name, indexes)
 
@@ -579,42 +599,65 @@ def _change_deleted_column_type_to_boolean_sqlite(engine, table_name,
                 column_copy = _get_not_supported_column(col_name_col_instance,
                                                         column.name)
             else:
-                column_copy = column.copy()
+                # FIXME(stephenfin): We shouldn't be using this private API;
+                # figure out how else to copy an arbitrary column schema
+                column_copy = column._copy()
         else:
             column_copy = Column('deleted', Boolean, default=0)
         columns.append(column_copy)
 
-    constraints = [constraint.copy() for constraint in table.constraints]
+    # FIXME(stephenfin): We shouldn't be using this private API;
+    # figure out how else to copy an arbitrary column schema
+    constraints = [constraint._copy() for constraint in table.constraints]
 
-    meta = table.metadata
-    new_table = Table(table_name + "__tmp__", meta,
-                      *(columns + constraints))
-    new_table.create()
+    with engine.connect() as conn:
+        meta = table.metadata
+        new_table = Table(
+            table_name + "__tmp__", meta,
+            *(columns + constraints))
 
-    indexes = []
-    for index in get_indexes(engine, table_name):
-        column_names = [new_table.c[c] for c in index['column_names']]
-        indexes.append(Index(index["name"], *column_names,
-                             unique=index["unique"]))
+        with conn.begin():
+            new_table.create(conn)
 
-    c_select = []
-    for c in table.c:
-        if c.name != "deleted":
-            c_select.append(c)
-        else:
-            c_select.append(table.c.deleted == table.c.id)
+        indexes = []
+        for index in get_indexes(engine, table_name):
+            column_names = [new_table.c[c] for c in index['column_names']]
+            indexes.append(
+                Index(index["name"], *column_names, unique=index["unique"])
+            )
 
-    table.drop()
-    for index in indexes:
-        index.create(engine)
+        c_select = []
+        for c in table.c:
+            if c.name != "deleted":
+                c_select.append(c)
+            else:
+                c_select.append(table.c.deleted == table.c.id)
 
-    new_table.rename(table_name)
-    new_table.update().\
-        where(new_table.c.deleted == new_table.c.id).\
-        values(deleted=True).\
-        execute()
+        with conn.begin():
+            table.drop(conn)
+            for index in indexes:
+                index.create(conn)
+
+        table.metadata.bind = engine
+        try:
+            new_table.rename(table_name)
+        finally:
+            table.metadata.bind = None
+
+        with conn.begin():
+            conn.execute(
+                new_table.update().where(
+                    new_table.c.deleted == new_table.c.id
+                ).values(deleted=True)
+            )
 
 
+@debtcollector.removals.remove(
+    message='This API is intended for use with sqlalchemy-migrate, support '
+    'for which is deprecated for removal; it will be removed in a future '
+    'release',
+    version='10.1.0',
+)
 def change_deleted_column_type_to_id_type(engine, table_name,
                                           **col_name_col_instance):
     if engine.name == "sqlite":
@@ -625,17 +668,28 @@ def change_deleted_column_type_to_id_type(engine, table_name,
 
     new_deleted = Column('new_deleted', table.c.id.type,
                          default=_get_default_deleted_value(table))
-    new_deleted.create(table, populate_default=True)
+    table.metadata.bind = engine
+    try:
+        new_deleted.create(table, populate_default=True)
+    finally:
+        table.metadata.bind = None
 
-    deleted = True  # workaround for pyflakes
-    table.update().\
-        where(table.c.deleted == deleted).\
-        values(new_deleted=table.c.id).\
-        execute()
-    table.c.deleted.drop()
-    table.c.new_deleted.alter(name="deleted")
+    table.metadata.bind = engine
+    try:
+        with engine.connect() as conn, conn.begin():
+            deleted = True  # workaround for pyflakes
+            conn.execute(
+                table.update().where(
+                    table.c.deleted == deleted
+                ).values(new_deleted=table.c.id)
+            )
 
-    _restore_indexes_on_deleted_columns(engine, table_name, indexes)
+        table.c.deleted.drop()
+        table.c.new_deleted.alter(name="deleted")
+
+        _restore_indexes_on_deleted_columns(engine, table_name, indexes)
+    finally:
+        table.metadata.bind = None
 
 
 def _is_deleted_column_constraint(constraint):
@@ -653,17 +707,17 @@ def _is_deleted_column_constraint(constraint):
 
 def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
                                                   **col_name_col_instance):
-    # NOTE(boris-42): sqlaclhemy-migrate can't drop column with check
-    #                 constraints in sqlite DB and our `deleted` column has
-    #                 2 check constraints. So there is only one way to remove
-    #                 these constraints:
-    #                 1) Create new table with the same columns, constraints
-    #                 and indexes. (except deleted column).
-    #                 2) Copy all data from old to new table.
-    #                 3) Drop old table.
-    #                 4) Rename new table to old table name.
-    meta = MetaData(bind=engine)
-    table = Table(table_name, meta, autoload=True)
+    # NOTE(boris-42): sqlalchemy-migrate can't drop column with check
+    # constraints in sqlite DB and our `deleted` column has two check
+    # constraints. There is only one way to remove these constraints:
+    #
+    # 1) Create new table with the same columns, constraints and indexes.
+    #    (except deleted column).
+    # 2) Copy all data from old to new table.
+    # 3) Drop old table.
+    # 4) Rename new table to old table name.
+    meta = MetaData()
+    table = Table(table_name, meta, autoload_with=engine)
     default_deleted_value = _get_default_deleted_value(table)
 
     columns = []
@@ -674,7 +728,9 @@ def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
                 column_copy = _get_not_supported_column(col_name_col_instance,
                                                         column.name)
             else:
-                column_copy = column.copy()
+                # FIXME(stephenfin): We shouldn't be using this private API;
+                # figure out how else to copy an arbitrary column schema
+                column_copy = column._copy()
         else:
             column_copy = Column('deleted', table.c.id.type,
                                  default=default_deleted_value)
@@ -683,35 +739,52 @@ def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
     constraints = []
     for constraint in table.constraints:
         if not _is_deleted_column_constraint(constraint):
-            constraints.append(constraint.copy())
+            # FIXME(stephenfin): We shouldn't be using this private API;
+            # figure out how else to copy an arbitrary constraint schema
+            constraints.append(constraint._copy())
 
-    new_table = Table(table_name + "__tmp__", meta,
-                      *(columns + constraints))
-    new_table.create()
+    with engine.connect() as conn:
+        # we need separate transactions, since we must create the table before
+        # we can copy entries into it (later)
+        with conn.begin():
+            new_table = Table(
+                table_name + "__tmp__", meta,
+                *(columns + constraints))
+            new_table.create(conn)
 
-    indexes = []
-    for index in get_indexes(engine, table_name):
-        column_names = [new_table.c[c] for c in index['column_names']]
-        indexes.append(Index(index["name"], *column_names,
-                             unique=index["unique"]))
+        indexes = []
+        for index in get_indexes(engine, table_name):
+            column_names = [new_table.c[c] for c in index['column_names']]
+            indexes.append(
+                Index(index["name"], *column_names, unique=index["unique"])
+            )
 
-    table.drop()
-    for index in indexes:
-        index.create(engine)
+        with conn.begin():
+            table.drop(conn)
+            for index in indexes:
+                index.create(conn)
 
-    new_table.rename(table_name)
-    deleted = True  # workaround for pyflakes
-    new_table.update().\
-        where(new_table.c.deleted == deleted).\
-        values(deleted=new_table.c.id).\
-        execute()
+        with conn.begin():
+            new_table.metadata.bind = engine
+            try:
+                new_table.rename(table_name)
+            finally:
+                new_table.metadata.bind = None
 
-    # NOTE(boris-42): Fix value of deleted column: False -> "" or 0.
-    deleted = False  # workaround for pyflakes
-    new_table.update().\
-        where(new_table.c.deleted == deleted).\
-        values(deleted=default_deleted_value).\
-        execute()
+            deleted = True  # workaround for pyflakes
+            conn.execute(
+                new_table.update().where(
+                    new_table.c.deleted == deleted
+                ).values(deleted=new_table.c.id)
+            )
+
+            # NOTE(boris-42): Fix value of deleted column: False -> "" or 0.
+            deleted = False  # workaround for pyflakes
+            conn.execute(
+                new_table.update().where(
+                    new_table.c.deleted == deleted
+                ).values(deleted=default_deleted_value)
+            )
 
 
 def get_db_connection_info(conn_pieces):
@@ -735,7 +808,7 @@ def get_indexes(engine, table_name):
     :param table_name: name of the table
     """
 
-    inspector = reflection.Inspector.from_engine(engine)
+    inspector = sqlalchemy.inspect(engine)
     indexes = inspector.get_indexes(table_name)
     return indexes
 
@@ -780,7 +853,7 @@ def add_index(engine, table_name, index_name, idx_columns):
         index = Index(
             index_name, *[getattr(table.c, col) for col in idx_columns]
         )
-        index.create()
+        index.create(engine)
     else:
         raise ValueError("Index '%s' already exists!" % index_name)
 
@@ -795,7 +868,7 @@ def drop_index(engine, table_name, index_name):
     table = get_table(engine, table_name)
     for index in table.indexes:
         if index.name == index_name:
-            index.drop()
+            index.drop(engine)
             break
     else:
         raise ValueError("Index '%s' not found!" % index_name)
@@ -1057,7 +1130,9 @@ def get_non_innodb_tables(connectable, skip_tables=('migrate_version',
 
     params['database'] = connectable.engine.url.database
     query = text(query_str)
-    noninnodb = connectable.execute(query, **params)
+    # TODO(stephenfin): What about if this is already a Connection?
+    with connectable.connect() as conn, conn.begin():
+        noninnodb = conn.execute(query, params)
     return [i[0] for i in noninnodb]
 
 
@@ -1167,164 +1242,22 @@ def suspend_fk_constraints_for_col_alter(
             ctx = MigrationContext.configure(conn)
             op = Operations(ctx)
 
-            for fk in fks:
-                op.drop_constraint(
-                    fk['name'], fk['source_table'], type_="foreignkey")
+            with conn.begin():
+                for fk in fks:
+                    op.drop_constraint(
+                        fk['name'], fk['source_table'], type_="foreignkey")
+
             yield
-            for fk in fks:
-                op.create_foreign_key(
-                    fk['name'], fk['source_table'],
-                    fk['referred_table'],
-                    fk['constrained_columns'],
-                    fk['referred_columns'],
-                    onupdate=fk['options'].get('onupdate'),
-                    ondelete=fk['options'].get('ondelete'),
-                    deferrable=fk['options'].get('deferrable'),
-                    initially=fk['options'].get('initially'),
-                )
 
-
-def getargspec(fn):
-    """Inspects a function for its argspec.
-
-    This is to handle a difference between py2/3. The Python 2.x getargspec
-    call is deprecated in Python 3.x, with the suggestion to use the signature
-    call instead.
-
-    To keep compatibility with the results, while avoiding deprecation
-    warnings, this instead will use the getfullargspec instead.
-
-    :param fn: The function to inspect.
-    :returns: The argspec for the function.
-    """
-    if hasattr(pyinspect, 'getfullargspec'):
-        return pyinspect.getfullargspec(fn)
-
-    return pyinspect.getargspec(fn)
-
-
-class NonCommittingConnectable(object):
-    """A ``Connectable`` substitute which rolls all operations back.
-
-    ``NonCommittingConnectable`` forms the basis of mock
-    ``Engine`` and ``Connection`` objects within a test.   It provides
-    only that part of the API that should reasonably be used within
-    a single-connection test environment (e.g. no engine.dispose(),
-    connection.invalidate(), etc. ).   The connection runs both within
-    a transaction as well as a savepoint.   The transaction is there
-    so that any operations upon the connection can be rolled back.
-    If the test calls begin(), a "pseduo" transaction is returned that
-    won't actually commit anything.   The subtransaction is there to allow
-    a test to successfully call rollback(), however, where all operations
-    to that point will be rolled back and the operations can continue,
-    simulating a real rollback while still remaining within a transaction
-    external to the test.
-
-    """
-
-    _nested_trans = None
-
-    def __init__(self, connection):
-        self.connection = connection
-        self._trans = connection.begin()
-        self._restart_nested()
-
-    def _restart_nested(self):
-        if self._nested_trans is not None:
-            self._nested_trans.rollback()
-        self._nested_trans = self.connection.begin_nested()
-
-    def _dispose(self):
-        if not self.connection.closed:
-            self._nested_trans.rollback()
-            self._trans.rollback()
-            self.connection.close()
-
-    def execute(self, obj, *multiparams, **params):
-        """Executes the given construct and returns a :class:`.ResultProxy`."""
-
-        return self.connection.execute(obj, *multiparams, **params)
-
-    def scalar(self, obj, *multiparams, **params):
-        """Executes and returns the first column of the first row."""
-
-        return self.connection.scalar(obj, *multiparams, **params)
-
-
-class NonCommittingEngine(NonCommittingConnectable):
-    """``Engine`` -specific non committing connectbale."""
-
-    @property
-    def url(self):
-        return self.connection.engine.url
-
-    @property
-    def engine(self):
-        return self
-
-    def connect(self):
-        return NonCommittingConnection(self.connection)
-
-    @contextlib.contextmanager
-    def begin(self):
-        conn = self.connect()
-        trans = conn.begin()
-        try:
-            yield conn
-        except Exception:
-            trans.rollback()
-        else:
-            trans.commit()
-
-
-class NonCommittingConnection(NonCommittingConnectable):
-    """``Connection`` -specific non committing connectbale."""
-
-    def close(self):
-        """Close the 'Connection'.
-
-        In this context, close() is a no-op.
-
-        """
-        pass
-
-    def begin(self):
-        return NonCommittingTransaction(self, self.connection.begin())
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, *arg):
-        pass
-
-
-class NonCommittingTransaction(object):
-    """A wrapper for ``Transaction``.
-
-    This is to accommodate being able to guaranteed start a new
-    SAVEPOINT when a transaction is rolled back.
-
-    """
-    def __init__(self, provisioned, transaction):
-        self.provisioned = provisioned
-        self.transaction = transaction
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, type, value, traceback):
-        if type is None:
-            try:
-                self.commit()
-            except Exception:
-                self.rollback()
-                raise
-        else:
-            self.rollback()
-
-    def commit(self):
-        self.transaction.commit()
-
-    def rollback(self):
-        self.transaction.rollback()
-        self.provisioned._restart_nested()
+            with conn.begin():
+                for fk in fks:
+                    op.create_foreign_key(
+                        fk['name'], fk['source_table'],
+                        fk['referred_table'],
+                        fk['constrained_columns'],
+                        fk['referred_columns'],
+                        onupdate=fk['options'].get('onupdate'),
+                        ondelete=fk['options'].get('ondelete'),
+                        deferrable=fk['options'].get('deferrable'),
+                        initially=fk['options'].get('initially'),
+                    )
