@@ -15,16 +15,16 @@ import contextlib
 import copy
 import fixtures
 import pickle
+import sys
+from unittest import mock
 import warnings
 
-import mock
 from oslo_config import cfg
 from oslo_context import context as oslo_context
-from oslotest import base as oslo_test_base
 from sqlalchemy import Column
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import registry
 from sqlalchemy.orm import Session
 from sqlalchemy import select
 from sqlalchemy import String
@@ -35,8 +35,9 @@ from oslo_db import options
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import engines as oslo_engines
 from oslo_db.sqlalchemy import orm
-from oslo_db.tests.sqlalchemy import base as test_base
-
+from oslo_db.tests import base as test_base
+from oslo_db.tests.sqlalchemy import base as db_test_base
+from oslo_db import warning
 
 enginefacade.transaction_context_provider(oslo_context.RequestContext)
 
@@ -90,7 +91,7 @@ class AssertDataSource(collections.namedtuple(
             assert False, "Unknown constant: %s" % const
 
 
-class MockFacadeTest(oslo_test_base.BaseTestCase):
+class MockFacadeTest(test_base.BaseTestCase):
     """test by applying mocks to internal call-points.
 
     This applies mocks to
@@ -135,15 +136,15 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             async_reader_maker = writer_maker
 
         if self.synchronous_reader:
-            reader_conn = async_reader_conn
-            reader_engine = async_reader_engine
-            reader_session = async_reader_session
-            reader_maker = async_reader_maker
-        else:
             reader_conn = writer_conn
             reader_engine = writer_engine
             reader_session = writer_session
             reader_maker = writer_maker
+        else:
+            reader_conn = async_reader_conn
+            reader_engine = async_reader_engine
+            reader_session = async_reader_session
+            reader_maker = async_reader_maker
 
         self.connections = AssertDataSource(
             writer_conn, reader_conn, async_reader_conn
@@ -236,9 +237,9 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             async_reader_engine = writer_engine
 
         if self.synchronous_reader:
-            reader_engine = async_reader_engine
-        else:
             reader_engine = writer_engine
+        else:
+            reader_engine = async_reader_engine
 
         engines = AssertDataSource(
             writer_engine, reader_engine, async_reader_engine)
@@ -333,9 +334,9 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             async_reader_maker = writer_maker
 
         if self.synchronous_reader:
-            reader_maker = async_reader_maker
-        else:
             reader_maker = writer_maker
+        else:
+            reader_maker = async_reader_maker
 
         makers = AssertDataSource(
             writer_maker,
@@ -478,6 +479,32 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             [mock.call.dispose()]
         )
 
+    def test_started_flag(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        self.assertTrue(facade.is_started)
+
+    def test_started_exception(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        exc = self.assertRaises(
+            enginefacade.AlreadyStartedError,
+            facade.configure,
+            connection=self.engine_uri
+        )
+        self.assertEqual(
+            "this TransactionFactory is already started",
+            exc.args[0]
+        )
+
     def test_session_reader_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -491,6 +518,19 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_reader_session(makers) as session:
                     session.execute("test")
 
+    def test_session_reader_decorator_kwarg_call(self):
+        context = oslo_context.RequestContext()
+
+        @enginefacade.reader
+        def go(context):
+            context.session.execute("test")
+        go(context=context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_reader_session(makers) as session:
+                    session.execute("test")
+
     def test_connection_reader_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -607,7 +647,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
 
     def test_async_on_writer_raises(self):
         exc = self.assertRaises(
-            TypeError, getattr, enginefacade.writer, "async"
+            TypeError, getattr, enginefacade.writer, "async_"
         )
         self.assertEqual(
             "Setting async on a WRITER makes no sense",
@@ -626,7 +666,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_reader_nested_in_async_reader_raises(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -647,7 +687,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_reader_allow_async_nested_in_async_reader(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -701,7 +741,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_writer_nested_in_async_reader_raises(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -741,10 +781,30 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_writer_session(makers) as session:
                     session.execute("test2")
 
+    def test_deprecated_async_reader_name(self):
+        if sys.version_info >= (3, 7):
+            self.skipTest("Test only runs on Python < 3.7")
+
+        context = oslo_context.RequestContext()
+
+        old = getattr(enginefacade.reader, "async")
+
+        @old
+        def go1(context):
+            context.session.execute("test1")
+
+        go1(context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_async_reader_session(
+                        makers, assert_calls=False) as session:
+                    session.execute("test1")
+
     def test_async_reader_then_reader_ok(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
 
@@ -1125,7 +1185,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                     session.execute("test")
 
 
-class PatchFactoryTest(oslo_test_base.BaseTestCase):
+class PatchFactoryTest(test_base.BaseTestCase):
 
     def test_patch_manager(self):
         normal_mgr = enginefacade.transaction_context()
@@ -1335,7 +1395,7 @@ class AsyncReaderWSlaveMockFacadeTest(MockFacadeTest):
     slave_uri = 'some_slave_connection'
 
 
-class LegacyIntegrationtest(test_base.DbTestCase):
+class LegacyIntegrationtest(db_test_base._DbTestCase):
 
     def test_legacy_integration(self):
         legacy_facade = enginefacade.get_legacy_facade()
@@ -1393,7 +1453,7 @@ class LegacyIntegrationtest(test_base.DbTestCase):
         )
 
 
-class ThreadingTest(test_base.DbTestCase):
+class ThreadingTest(db_test_base._DbTestCase):
     """Test copy/pickle on new threads using real connections and sessions."""
 
     def _assert_ctx_connection(self, context, connection):
@@ -1586,7 +1646,7 @@ class ThreadingTest(test_base.DbTestCase):
                 assert session is not session2
 
 
-class LiveFacadeTest(test_base.DbTestCase):
+class LiveFacadeTest(db_test_base._DbTestCase):
     """test using live SQL with test-provisioned databases.
 
     Several of these tests require that multiple transactions run
@@ -1611,11 +1671,13 @@ class LiveFacadeTest(test_base.DbTestCase):
         metadata.create_all(self.engine)
         self.addCleanup(metadata.drop_all, self.engine)
 
+        reg = registry()
+
         class User(object):
             def __init__(self, name):
                 self.name = name
 
-        mapper(User, user_table)
+        reg.map_imperatively(User, user_table)
         self.User = User
 
     def _assert_ctx_connection(self, context, connection):
@@ -1748,7 +1810,7 @@ class LiveFacadeTest(test_base.DbTestCase):
             session.query(self.User.name).scalar()
         )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_session_transaction(self):
         context = oslo_context.RequestContext()
         with enginefacade.writer.using(context) as session:
@@ -1780,7 +1842,7 @@ class LiveFacadeTest(test_base.DbTestCase):
 
         session = self.sessionmaker(autocommit=True)
 
-        # inner transction + second part of "outer" transaction were committed
+        # inner transaction + second part of "outer" transaction were committed
         self.assertEqual(
             [("u2",), ("u3",), ("u4", )],
             session.query(
@@ -1820,7 +1882,7 @@ class LiveFacadeTest(test_base.DbTestCase):
 
         session = self.sessionmaker(autocommit=True)
 
-        # inner transction + second part of "outer" transaction were committed
+        # inner transaction + second part of "outer" transaction were committed
         self.assertEqual(
             [("u1",), ("u3",), ("u4", )],
             session.query(
@@ -1848,14 +1910,14 @@ class LiveFacadeTest(test_base.DbTestCase):
 
         session = self.sessionmaker(autocommit=True)
 
-        # inner transction + second part of "outer" transaction were committed
+        # inner transaction + second part of "outer" transaction were committed
         self.assertEqual(
             [("u1",), ("u3",), ("u4", )],
             session.query(
                 self.User.name).order_by(self.User.name).all()
         )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_session_transaction_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -1896,14 +1958,14 @@ class LiveFacadeTest(test_base.DbTestCase):
 
         session = self.sessionmaker(autocommit=True)
 
-        # inner transction + second part of "outer" transaction were committed
+        # inner transaction + second part of "outer" transaction were committed
         self.assertEqual(
             [("u2",), ("u3",), ("u4", )],
             session.query(
                 self.User.name).order_by(self.User.name).all()
         )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_connection_transaction(self):
         context = oslo_context.RequestContext()
         with enginefacade.writer.connection.using(context) as connection:
@@ -1940,11 +2002,11 @@ class LiveFacadeTest(test_base.DbTestCase):
                 self.User.name).order_by(self.User.name).all()
         )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_writer_in_reader(self):
         context = oslo_context.RequestContext()
         with enginefacade.reader.using(context) as session:
-            ping = session.scalar(select([1]))
+            ping = session.scalar(select(1))
             self.assertEqual(1, ping)
 
             # we're definitely a reader
@@ -2127,16 +2189,18 @@ class LiveFacadeTest(test_base.DbTestCase):
 
 
 class MySQLLiveFacadeTest(
-        test_base.MySQLOpportunisticTestCase, LiveFacadeTest):
+    db_test_base._MySQLOpportunisticTestCase, LiveFacadeTest,
+):
     pass
 
 
 class PGLiveFacadeTest(
-        test_base.PostgreSQLOpportunisticTestCase, LiveFacadeTest):
+    db_test_base._PostgreSQLOpportunisticTestCase, LiveFacadeTest,
+):
     pass
 
 
-class ConfigOptionsTest(oslo_test_base.BaseTestCase):
+class ConfigOptionsTest(test_base.BaseTestCase):
     def test_all_options(self):
         """test that everything in CONF.database.iteritems() is accepted.
 
@@ -2161,7 +2225,7 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
 
         self.assertEqual(1, len(w))
         self.assertTrue(
-            issubclass(w[-1].category, exception.NotSupportedWarning))
+            issubclass(w[-1].category, warning.NotSupportedWarning))
         self.assertEqual(
             "Configuration option(s) ['fake1', 'wrong2'] not supported",
             str(w[-1].message)
@@ -2181,7 +2245,7 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
         )
 
 
-class TestTransactionFactoryCallback(oslo_test_base.BaseTestCase):
+class TestTransactionFactoryCallback(test_base.BaseTestCase):
 
     def test_setup_for_connection_called_with_profiler(self):
         context_manager = enginefacade.transaction_context()
