@@ -23,12 +23,12 @@ from unittest import mock
 
 import fixtures
 from oslo_config import cfg
-from oslotest import base as oslo_test
 import sqlalchemy
+from sqlalchemy import sql
 from sqlalchemy import Column, MetaData, Table
 from sqlalchemy.engine import url
 from sqlalchemy import Integer, String
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import declarative_base
 
 from oslo_db import exception
 from oslo_db import options as db_options
@@ -36,7 +36,8 @@ from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session
-from oslo_db.tests.sqlalchemy import base as test_base
+from oslo_db.tests import base as test_base
+from oslo_db.tests.sqlalchemy import base as db_test_base
 
 
 BASE = declarative_base()
@@ -51,23 +52,22 @@ class RegexpTable(BASE, models.ModelBase):
     bar = Column(String(255))
 
 
-class RegexpFilterTestCase(test_base._DbTestCase):
+class RegexpFilterTestCase(db_test_base._DbTestCase):
 
     def setUp(self):
         super(RegexpFilterTestCase, self).setUp()
         meta = MetaData()
-        meta.bind = self.engine
         test_table = Table(_REGEXP_TABLE_NAME, meta,
                            Column('id', Integer, primary_key=True,
                                   nullable=False),
                            Column('bar', String(255)))
-        test_table.create()
-        self.addCleanup(test_table.drop)
+        test_table.create(self.engine)
+        self.addCleanup(test_table.drop, self.engine)
 
     def _test_regexp_filter(self, regexp, expected):
-        with enginefacade.writer.using(test_base.context):
-            _session = test_base.context.session
-            for i in ['10', '20', u'♥']:
+        with enginefacade.writer.using(db_test_base.context):
+            _session = db_test_base.context.session
+            for i in ['10', '20', '♥']:
                 tbl = RegexpTable()
                 tbl.update({'bar': i})
                 tbl.save(session=_session)
@@ -83,13 +83,13 @@ class RegexpFilterTestCase(test_base._DbTestCase):
         self._test_regexp_filter('11', [])
 
     def test_regexp_filter_unicode(self):
-        self._test_regexp_filter(u'♥', [u'♥'])
+        self._test_regexp_filter('♥', ['♥'])
 
     def test_regexp_filter_unicode_nomatch(self):
-        self._test_regexp_filter(u'♦', [])
+        self._test_regexp_filter('♦', [])
 
 
-class SQLiteSavepointTest(test_base._DbTestCase):
+class SQLiteSavepointTest(db_test_base._DbTestCase):
     def setUp(self):
         super(SQLiteSavepointTest, self).setUp()
         meta = MetaData()
@@ -109,16 +109,22 @@ class SQLiteSavepointTest(test_base._DbTestCase):
         )
         self.assertEqual(
             [(1, 'data 1')],
-            self.engine.execute(
+            conn.execute(
                 self.test_table.select().
                 order_by(self.test_table.c.id)
             ).fetchall()
         )
         trans.rollback()
-        self.assertEqual(
-            0,
-            self.engine.scalar(self.test_table.count())
-        )
+
+        with self.engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.scalar(
+                    sqlalchemy.select(
+                        sqlalchemy.func.count(self.test_table.c.id),
+                    ).select_from(self.test_table)
+                )
+            )
 
     def test_savepoint_middle(self):
         with self.engine.begin() as conn:
@@ -139,13 +145,13 @@ class SQLiteSavepointTest(test_base._DbTestCase):
                 {'data': 'data 3'}
             )
 
-        self.assertEqual(
-            [(1, 'data 1'), (2, 'data 3')],
-            self.engine.execute(
-                self.test_table.select().
-                order_by(self.test_table.c.id)
-            ).fetchall()
-        )
+            self.assertEqual(
+                [(1, 'data 1'), (2, 'data 3')],
+                conn.execute(
+                    self.test_table.select().
+                    order_by(self.test_table.c.id)
+                ).fetchall()
+            )
 
     def test_savepoint_beginning(self):
         with self.engine.begin() as conn:
@@ -161,13 +167,13 @@ class SQLiteSavepointTest(test_base._DbTestCase):
                 {'data': 'data 2'}
             )
 
-        self.assertEqual(
-            [(1, 'data 2')],
-            self.engine.execute(
-                self.test_table.select().
-                order_by(self.test_table.c.id)
-            ).fetchall()
-        )
+            self.assertEqual(
+                [(1, 'data 2')],
+                conn.execute(
+                    self.test_table.select().
+                    order_by(self.test_table.c.id)
+                ).fetchall()
+            )
 
 
 class FakeDBAPIConnection(object):
@@ -196,23 +202,7 @@ class ProgrammingError(Exception):
     pass
 
 
-class FakeDB2Engine(object):
-
-    class Dialect(object):
-
-        def is_disconnect(self, e, *args):
-            expected_error = ('SQL30081N: DB2 Server connection is no longer '
-                              'active')
-            return (str(e) == expected_error)
-
-    dialect = Dialect()
-    name = 'ibm_db_sa'
-
-    def dispose(self):
-        pass
-
-
-class QueryParamTest(test_base.DbTestCase):
+class QueryParamTest(db_test_base.DbTestCase):
     def _fixture(self):
         from sqlalchemy import create_engine
 
@@ -223,6 +213,16 @@ class QueryParamTest(test_base.DbTestCase):
             "oslo_db.sqlalchemy.engines.sqlalchemy.create_engine",
             side_effect=_mock_create_engine)
 
+    def _normalize_query_dict(self, qdict):
+        # SQLAlchemy 1.4 returns url.query as:
+        # immutabledict({k1: v1, k2: (v2a, v2b, ...), ...})
+        # that is with tuples not lists for multiparams
+
+        return {
+            k: list(v) if isinstance(v, tuple) else v
+            for k, v in qdict.items()
+        }
+
     def test_add_assorted_params(self):
         with self._fixture() as ce:
             engines.create_engine(
@@ -230,7 +230,7 @@ class QueryParamTest(test_base.DbTestCase):
                 connection_parameters="foo=bar&bat=hoho&bat=param2")
 
         self.assertEqual(
-            ce.mock_calls[0][1][0].query,
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
             {'bat': ['hoho', 'param2'], 'foo': 'bar'}
         )
 
@@ -241,7 +241,7 @@ class QueryParamTest(test_base.DbTestCase):
 
         self.assertEqual(
             ce.mock_calls[0][1][0].query,
-            {}
+            self._normalize_query_dict({})
         )
 
     def test_combine_params(self):
@@ -254,7 +254,7 @@ class QueryParamTest(test_base.DbTestCase):
                                       "bind_host=192.168.1.5")
 
         self.assertEqual(
-            ce.mock_calls[0][1][0].query,
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
             {
                 'bind_host': '192.168.1.5',
                 'charset': 'utf8',
@@ -274,7 +274,7 @@ class QueryParamTest(test_base.DbTestCase):
                                       "bind_host=192.168.1.5")
 
         self.assertEqual(
-            ce.mock_calls[0][1][0].query,
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
             {
                 'bind_host': '192.168.1.5',
                 'charset': 'utf8',
@@ -285,17 +285,17 @@ class QueryParamTest(test_base.DbTestCase):
         )
 
 
-class MySQLDefaultModeTestCase(test_base._MySQLOpportunisticTestCase):
+class MySQLDefaultModeTestCase(db_test_base._MySQLOpportunisticTestCase):
     def test_default_is_traditional(self):
         with self.engine.connect() as conn:
             sql_mode = conn.execute(
-                "SHOW VARIABLES LIKE 'sql_mode'"
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
             ).first()[1]
 
         self.assertIn("TRADITIONAL", sql_mode)
 
 
-class MySQLModeTestCase(test_base._MySQLOpportunisticTestCase):
+class MySQLModeTestCase(db_test_base._MySQLOpportunisticTestCase):
 
     def __init__(self, *args, **kwargs):
         super(MySQLModeTestCase, self).__init__(*args, **kwargs)
@@ -314,20 +314,22 @@ class MySQLModeTestCase(test_base._MySQLOpportunisticTestCase):
         self.test_table = Table(_TABLE_NAME + "mode", meta,
                                 Column('id', Integer, primary_key=True),
                                 Column('bar', String(255)))
-        self.test_table.create(self.connection)
+        with self.connection.begin():
+            self.test_table.create(self.connection)
 
         def cleanup():
-            self.test_table.drop(self.connection)
+            with self.connection.begin():
+                self.test_table.drop(self.connection)
             self.connection.close()
             mode_engine.dispose()
+
         self.addCleanup(cleanup)
 
     def _test_string_too_long(self, value):
         with self.connection.begin():
-            self.connection.execute(self.test_table.insert(),
-                                    bar=value)
+            self.connection.execute(self.test_table.insert(), {'bar': value})
             result = self.connection.execute(self.test_table.select())
-            return result.fetchone()['bar']
+            return result.fetchone().bar
 
     def test_string_too_long(self):
         value = 'a' * 512
@@ -364,7 +366,7 @@ class MySQLTraditionalModeTestCase(MySQLStrictAllTablesModeTestCase):
         self.mysql_mode = 'TRADITIONAL'
 
 
-class EngineFacadeTestCase(oslo_test.BaseTestCase):
+class EngineFacadeTestCase(test_base.BaseTestCase):
     def setUp(self):
         super(EngineFacadeTestCase, self).setUp()
 
@@ -388,10 +390,6 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         self.assertFalse(ses.autocommit)
         self.assertTrue(ses.expire_on_commit)
 
-    def test_direct_invocation_deprecated_args(self):
-        facade = session.EngineFacade("sqlite://", idle_timeout=59)
-        self.assertEqual(59, facade.get_engine().pool._recycle)
-
     @mock.patch('oslo_db.sqlalchemy.orm.get_maker')
     @mock.patch('oslo_db.sqlalchemy.engines.create_engine')
     def test_creation_from_config(self, create_engine, get_maker):
@@ -474,50 +472,61 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         self.assertEqual(master_path, str(slave_session.bind.url))
 
 
-class SQLiteConnectTest(oslo_test.BaseTestCase):
+class SQLiteConnectTest(test_base.BaseTestCase):
 
     def _fixture(self, **kw):
         return session.create_engine("sqlite://", **kw)
 
     def test_sqlite_fk_listener(self):
         engine = self._fixture(sqlite_fk=True)
-        self.assertEqual(
-            1,
-            engine.scalar("pragma foreign_keys")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                1,
+                conn.execute(
+                    sql.text('pragma foreign_keys')
+                ).scalars().first(),
+            )
 
         engine = self._fixture(sqlite_fk=False)
 
-        self.assertEqual(
-            0,
-            engine.scalar("pragma foreign_keys")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.execute(
+                    sql.text('pragma foreign_keys')
+                ).scalars().first(),
+            )
 
     def test_sqlite_synchronous_listener(self):
         engine = self._fixture()
 
         # "The default setting is synchronous=FULL." (e.g. 2)
         # http://www.sqlite.org/pragma.html#pragma_synchronous
-        self.assertEqual(
-            2,
-            engine.scalar("pragma synchronous")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                2,
+                conn.execute(sql.text('pragma synchronous')).scalars().first(),
+            )
 
         engine = self._fixture(sqlite_synchronous=False)
 
-        self.assertEqual(
-            0,
-            engine.scalar("pragma synchronous")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.execute(sql.text('pragma synchronous')).scalars().first(),
+            )
 
 
-class MysqlConnectTest(test_base._MySQLOpportunisticTestCase):
+class MysqlConnectTest(db_test_base._MySQLOpportunisticTestCase):
 
     def _fixture(self, sql_mode):
         return session.create_engine(self.engine.url, mysql_sql_mode=sql_mode)
 
     def _assert_sql_mode(self, engine, sql_mode_present, sql_mode_non_present):
-        mode = engine.execute("SHOW VARIABLES LIKE 'sql_mode'").fetchone()[1]
+        with engine.connect() as conn:
+            mode = conn.execute(
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
+            ).fetchone()[1]
         self.assertIn(
             sql_mode_present, mode
         )
@@ -541,8 +550,10 @@ class MysqlConnectTest(test_base._MySQLOpportunisticTestCase):
         # get the GLOBAL sql_mode, not the @@SESSION, so that
         # we get what is configured for the MySQL database, as opposed
         # to what our own session.create_engine() has set it to.
-        expected = self.engine.execute(
-            "SELECT @@GLOBAL.sql_mode").scalar()
+        with self.engine.connect() as conn:
+            expected = conn.execute(
+                sql.text("SELECT @@GLOBAL.sql_mode")
+            ).scalar()
 
         engine = self._fixture(sql_mode=None)
         self._assert_sql_mode(engine, expected, None)
@@ -594,8 +605,10 @@ class MysqlConnectTest(test_base._MySQLOpportunisticTestCase):
 
         engine = self._fixture(sql_mode='TRADITIONAL')
 
-        actual_mode = engine.execute(
-            "SHOW VARIABLES LIKE 'sql_mode'").fetchone()[1]
+        with engine.connect() as conn:
+            actual_mode = conn.execute(
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
+            ).fetchone()[1]
 
         self.assertIn('MySQL server mode set to %s' % actual_mode,
                       log.output)
@@ -631,7 +644,7 @@ class MysqlConnectTest(test_base._MySQLOpportunisticTestCase):
                          log.output)
 
 
-class CreateEngineTest(oslo_test.BaseTestCase):
+class CreateEngineTest(test_base.BaseTestCase):
     """Test that dialect-specific arguments/ listeners are set up correctly.
 
     """
@@ -745,7 +758,7 @@ class CreateEngineTest(oslo_test.BaseTestCase):
         def warn_interpolate(msg, args):
             # test the interpolation itself to ensure the password
             # is concealed
-            warnings.warning(msg % args)
+            warnings.warning(msg % (args, ))
 
         with mock.patch(
                 "oslo_db.sqlalchemy.engines.LOG.warning",
@@ -785,7 +798,7 @@ class CreateEngineTest(oslo_test.BaseTestCase):
         )
 
 
-class ProcessGuardTest(test_base._DbTestCase):
+class ProcessGuardTest(db_test_base._DbTestCase):
     def test_process_guard(self):
         self.engine.dispose()
 
@@ -813,7 +826,7 @@ class ProcessGuardTest(test_base._DbTestCase):
         self.assertEqual(new_dbapi_id, newer_dbapi_id)
 
 
-class PatchStacktraceTest(test_base._DbTestCase):
+class PatchStacktraceTest(db_test_base._DbTestCase):
 
     def test_trace(self):
         engine = self.engine
@@ -836,7 +849,7 @@ class PatchStacktraceTest(test_base._DbTestCase):
             with mock.patch.object(engine.dialect, "do_execute") as mock_exec:
 
                 mock_exec.side_effect = orig_do_exec
-                conn.execute("select 1;")
+                conn.execute(sql.text("select 1"))
 
             call = mock_exec.mock_calls[0]
 
